@@ -735,7 +735,7 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(opts.http_timeout, '20')
         with testtools.ExpectedException(AttributeError):
             opts.os_network_service_type
-        cloud = c.get_one_cloud(argparse=opts, verify=False)
+        cloud = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cloud.config['service_type'], 'network')
         self.assertEqual(cloud.config['interface'], 'admin')
         self.assertEqual(cloud.config['api_timeout'], '20')
@@ -753,10 +753,10 @@ class TestConfigArgparse(base.TestCase):
         opts, _remain = parser.parse_known_args(args)
         self.assertEqual(opts.os_service_type, 'network')
         self.assertEqual(opts.os_endpoint_type, 'admin')
-        self.assertEqual(opts.os_network_service_type, None)
-        self.assertEqual(opts.os_network_api_version, None)
+        self.assertIsNone(opts.os_network_service_type)
+        self.assertIsNone(opts.os_network_api_version)
         self.assertEqual(opts.network_api_version, '4')
-        cloud = c.get_one_cloud(argparse=opts, verify=False)
+        cloud = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cloud.config['service_type'], 'network')
         self.assertEqual(cloud.config['interface'], 'admin')
         self.assertEqual(cloud.config['network_api_version'], '4')
@@ -776,14 +776,14 @@ class TestConfigArgparse(base.TestCase):
             parser, args, ['compute', 'network', 'volume'])
         opts, _remain = parser.parse_known_args(args)
         self.assertEqual(opts.os_network_service_type, 'badtype')
-        self.assertEqual(opts.os_compute_service_type, None)
-        self.assertEqual(opts.os_volume_service_type, None)
+        self.assertIsNone(opts.os_compute_service_type)
+        self.assertIsNone(opts.os_volume_service_type)
         self.assertEqual(opts.os_service_type, 'compute')
         self.assertEqual(opts.os_compute_service_name, 'cloudServers')
         self.assertEqual(opts.os_endpoint_type, 'admin')
-        self.assertEqual(opts.os_network_api_version, None)
+        self.assertIsNone(opts.os_network_api_version)
         self.assertEqual(opts.network_api_version, '4')
-        cloud = c.get_one_cloud(argparse=opts, verify=False)
+        cloud = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cloud.config['service_type'], 'compute')
         self.assertEqual(cloud.config['network_service_type'], 'badtype')
         self.assertEqual(cloud.config['interface'], 'admin')
